#ifndef _MISTCLIENT_H_
#define _MISTCLIENT_H_

#include "../zerofpsv2/engine/application.h"
#include "../zerofpsv2/gui/zgui.h"
#include "pointtext.h"

class OptionsDlg;
class ActionMenu;
class InventoryDlg;
class EquipmentDlg;
class SkillBar;
class Camera;
class Entity;

typedef void (*msgScreenProg)(string, string, unsigned int msg, int numparms, void *params);

enum GUI_SCRIPT
{
	GSF_GAMEGUI,
	GSF_OPTION,
	GSF_CHARGEN,
	GSF_START,
	GSF_ITEMINFO,
};

enum HUD_EFFECTS
{
	HUD_WATER,
	HUD_DEAD,
	HUD_FOG,

};


const int MAX_NUM_BUFF_ICONS = 32;


/**	\brief	Da MistClient
		\ingroup MistClient
*/
class MistClient :public Application, public ZGuiApp {
	private:
		//console functions
		enum FuncId_e
		{
			FID_SAY,
			FID_PLAYERLIST,
			FID_KILLME,
			FID_MSREFRESH,
		};

		//music
		string		m_strMenuMusic;
		
		bool			m_bBloom;
		
		//point text handler
		PointText*	m_pkPointText;
		
		//delay
		float			m_fDelayTime;

		//character and camera
		Camera*		m_pkCamera;						//main camera
		bool			m_bFrontView;				
		int			m_iCharacterID;				//current active character ID      
		int			m_iPickedEntityID;			// Selected entity for use.								
		bitset<6>	m_kCharacterControls;
		bool			m_bDead;
		bool			m_bCombatMode;
		
		//game
		vector<string>	m_kEmotes;					//emote list
		int				m_iTargetID;
		
		Entity*		m_pkMakerEntity;
		
		//states
		bool			m_bShowLagMeeter;
		bool			m_bTargetRotate;
		bool			m_bShowMenulevel;
		bool			m_bQuickStart; 
		string		m_strQuickStartAddress;	
		bool			m_bGuiCapture;
		string		m_strLoginName, m_strLoginPW;
		bool			m_bLoginKeepAlive;			// If we are in login menu we send to keep connection open.
		float			m_fPingDelay;

		vector<string>	m_kPlayerList;		//list of players since last playerlist update

		//gui
		map<GUI_SCRIPT, string> m_kGuiScrips;			
		vector<pair<string,string> > 	m_kServerList;
		map<string, msgScreenProg> 	m_kGuiMsgProcs;

		OptionsDlg* 	m_pkOptionsDlg;
		ActionMenu* 	m_pkActionDlg;
		InventoryDlg* 	m_pkInventoryDlg;
		EquipmentDlg* 	m_pkEquipmentDlg;
		SkillBar*		m_pkSkillBar;


		struct BUFF_ICON_INFO
		{
			string m_strName;
			string m_strIcon;
			float  m_fTimeout;
			float  m_fTimeLeft;
			float  m_fStartTime;
			char   m_cType;	
			ZGuiLabel* m_pkWnd;
			ZGuiProgressbar* m_pkProgressBar;
		};

		void InitBuffWnd();
		void RebuildBuffIconList(vector<BUFF_ICON_INFO>* kList);
		void UpdateBuffIconList();
		
		int m_iNumBuffIcons;
		BUFF_ICON_INFO m_kBuffIcons[MAX_NUM_BUFF_ICONS];

		bool ReadWriteServerList(bool bRead);

		void UpdateCharacter();
		void SendControlInfo();
		void Input();
		bool DelayCommand();
		
		
		//chat system
		void Say(string strMsg);
		void RegisterEmotes();
		
		//specifik network requests
		void RequestOpenInventory();
		void RequestPickup(int iEntityID,int iPosX = -1,int iPosY = -1);
		
		//general network sends
		void SendUseItem(int iItemID);
		void SendSetDefaultAttack(const string& strSkill);		
		void SendAddSkillToQueue(const string& strSkill,int iTargetID);		
		void SendAddStatPoint(int iStat);
		void SendAddSkillToSkillbar(const string& strSkill,int iPos);
		void SendRemoveItemFromSkillbar(int iPos);
		void SendSit();
		void SendSetTarget(int iTargetID);
		void SendCombatMode(bool bCombatMode);
		void SendAction(int iEntityID,const string& strAction);
		void SendRequestContainer(int iContainerID);		
		void SendMoveItem(int iItemID,int iTarget,int iPosX,int iPosY,int iCount = -1);	
		void SendRequestPlayerList();
		void SendMessage(string strMsg,int iChannel,string strToWho);
		void SendRequestKillMe();
		void SendRequestOpenEquipment();			
		void SendTaunt(int iID);
		void SendTaunt(const string& strEmote);
		void SendRespawnRequest();
		void RequestItemInfo(int iItemID);
		void SendUseSkill(const string& strSkill,int iTargetID,const Vector3& kPos,const Vector3& kDir);		
			/*SendMoveItem hj?p.
				iItemID : alltid entity id't f? itemet i fr?a
				iTraget : target container entity ID , om -1 s?antas i spelarens inventory
				iPosX   : position i containern som f?em?et skall flyttas till, om -1 s?s?ts f?em?et p?en ledig plats
			*/
		
		
			
		// gui stuff
		void ShowLag();
		void SetupGUI();
		void InitGUIFonts();
		void FindGUIScriptsByResSuffix();
		bool NameIPFromServerList(string& strName, string& strIP);
		void ToogleChatWnd(bool bOpen, bool bSetInputFocus=false);
		void ToggleEmoteList(bool bOpen);
		//void ResizeChatDlg(bool bBigger);      	
		void LoadInGameGui();
		void LoadStartScreenGui(bool bShowSplashImage);
		void SetGuiCapture(bool bSet, bool bMoveCursorToCenter=true);
		void PositionActionButtons();
		void CloseActiveContainer();
		void CloseActiveWindow(); // on ESC press
						
		//register
		void RegisterPropertys();
		void RegisterResources();
	  
		//draw
		void DrawCrossHair();
		void DrawTargetMarker();
		void DrawMouseOverMarker(const Vector3& kPos,float fSize);
		void DrawHUDEffect(int iHUDEffect);
		
		//picking
		Entity* GetTargetObject();
		Vector3 Get3DMouseDir(bool bMouse);

				
	public:
		//application virtuals
		MistClient(char* aName,int iWidth,int iHeight,int iDepth);
		
		bool StartUp();
		bool ShutDown();
		bool IsValid()			{return true;}
		void OnInit(void);
		void OnIdle(void);
		void OnHud(void);
		void OnSystem();
		void RenderInterface(void);

		void RunCommand(int cmdid, const CmdArgument* kCommand);
				
		void OnClientStart(void);			
		void OnClientConnected();
		void OnNetworkMessage(NetPacket *PkNetMessage);
		void OnDisconnect(int iConnectionID);

		void OnSystemMessage(const string& strType,int iNrOfParam,const void** pkParams);

      void AddRemoveServer(const char* szName, const char* szSeverIP, bool bAdd=true);
      void UpdateServerListbox();
		void AddStringToChatBox(string strMsg);

		void RegBeginPlay(string strChar);
		void DeleteChar(string strChar);
		void AddChar(string strChar, string strMod);

		
		int	GetCurrentTargetID()														{	return m_iTargetID;		}
		bool	GetCombatMode()															{	return m_bCombatMode;	}
		
		
      friend bool GUIPROC( ZGuiWnd* win, unsigned int msg, int numparms, void *params );
      friend void GuiMsgStartScreen( string strMainWnd, string strController, unsigned int msg, int numparms, void *params );
		friend void GuiMsgIngameScreen( string strMainWnd, string strController, unsigned int msg, int numparms, void *params );
		friend void GuiMsgOptionsDlg( string strMainWnd, string strController, unsigned int msg, int numparms, void *params );
		friend void GuiMsgActionDlg( string strMainWnd, string strController, unsigned int msg, int numparms, void *params );
		friend void GuiMsgInventoryDlg( string strMainWnd, string strController, unsigned int msg, int numparms, void *params );
		friend void GuiMsgEquipmentDlg( string strMainWnd, string strController, unsigned int msg, int numparms, void *params );
		
		friend class OptionsDlg;
		friend class ActionMenu;
		friend class InventoryDlg;
		friend class EquipmentDlg;
		friend class SkillBar;
};



#endif











